@@ -209,6 +209,23 @@ try:
 except:
     pass
 
+#==============================================================================
+# Turtle adjustments
+#==============================================================================
+# This is needed to prevent turtle scripts crashes after multiple runs in the
+# same IPython Console instance.
+# See Spyder issue #6278
+import turtle
+from turtle import Screen, Terminator
+
+def spyder_bye():
+    try:
+        Screen().bye()
+        turtle.TurtleScreen._RUNNING = True
+    except Terminator:
+        pass
+turtle.bye = spyder_bye
+
 
 #==============================================================================
 # Pandas adjustments
@@ -555,16 +572,34 @@ class UserModuleReloader(object):
         Add to pathlist Python library paths to be skipped from module
         reloading.
         """
+        # Get standard installation paths
         try:
             paths = sysconfig.get_paths()
-            lib_paths = [paths['stdlib'],
-                         paths['purelib'],
-                         paths['scripts'],
-                         paths['data']]
-
-            return initial_pathlist + lib_paths
+            standard_paths = [paths['stdlib'],
+                              paths['purelib'],
+                              paths['scripts'],
+                              paths['data']]
         except Exception:
-            return initial_pathlist
+            standard_paths = []
+
+        # Get user installation path
+        # See Spyder issue 8776
+        try:
+            import site
+            if getattr(site, 'getusersitepackages', False):
+                # Virtualenvs don't have this function but
+                # conda envs do
+                user_path = [site.getusersitepackages()]
+            elif getattr(site, 'USER_SITE', False):
+                # However, it seems virtualenvs have this
+                # constant
+                user_path = [site.USER_SITE]
+            else:
+                user_path = []
+        except Exception:
+            user_path = []
+
+        return initial_pathlist + standard_paths + user_path
 
     def is_module_reloadable(self, module, modname):
         """Decide if a module is reloadable or not."""
@@ -599,12 +634,15 @@ class UserModuleReloader(object):
             return True
         elif not os.name == 'nt':
             # Module paths containing the strings below can be ihherited
-            # from the default Linux installation or Homebrew in a
-            # virtualenv.
+            # from the default Linux installation, Homebrew or the user
+            # site-packages in a virtualenv.
             patterns = [r'^/usr/lib.*',
                         r'^/usr/local/lib.*',
                         r'^/usr/.*/dist-packages/.*',
-                        r'^/Library/.*'
+                        r'^/home/.*/.local/lib.*',
+                        r'^/Library/.*',
+                        r'^/Users/.*/Library/.*',
+                        r'^/Users/.*/.local/.*',
             ]
 
             if [p for p in patterns if re.search(p, modpath)]:
@@ -787,7 +825,13 @@ def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
 
     clear_post_mortem()
     sys.argv = ['']
-    namespace.pop('__file__')
+
+    # Avoid error when running `%reset -f` programmatically
+    # See issue spyder-ide/spyder-kernels#91
+    try:
+        namespace.pop('__file__')
+    except KeyError:
+        pass
 
 builtins.runfile = runfile
 
